    
/**
 * @file
 * @details This file contains functions related to the LCD. All advanced features
 * related to printing to the LCD, moving the cursor or clearing LCD are
 * stored here. Also backlight on/off function are stored.
 *
 * @defgroup lcd LCD
 * @ingroup slave
 * @{
 */

#include "../include/include.h"

extern UINT8 DutyCycle;

const char line_address[] = {ST7032_ADDRESS_LINE_1,ST7032_ADDRESS_LINE_2};

/**
 * This function writes to the LCD screen.
 */
void SPI_TX(UINT8 Command)
{
    UINT8 rx;


    LCD_CS_OUT  = LCD_SELECTED;
    __delay_us(10);

    spi_write(Command);

    LCD_CS_OUT  = LCD_nSELECTED;
    __delay_us(10);
}



/**
 * This function points to an LCD position.
 */
void lcd_goto(UINT8 x,UINT8 y)
{
    lcd_putcmd(ST7032_DDRAM_ADDRESS(line_address[y] + x));
}

/**
 * This function clears the LCD screen.
 */
void lcd_clear(void)
{
    lcd_putcmd(ST7032_CLEAR);   //clear
    __delay_ms(1);
}

/**
 * This function sends an LCD message.
 */
void lcd_putline(const UINT8 l, const char *buffer)
{
    lcd_putcmd(line_address[l]);
    lcd_putstr(buffer);
}

/**
 * This function writes a string to the LCD.
 */
void lcd_putstr(const char *s)
{
    while(*s)              
    {
        lcd_putchar(*s++);
    }
}

/**
 * This function initialises the LCD.
 */
void lcd_init(void)
{

   // Configure the controller
   LCD_RST_DIR   = 0;
   LCD_CS_DIR    = 0;
   LCD_RS_DIR    = 0;

   LCD_SDI_DIR   = 0;
   LCD_SCL_DIR   = 0;
   
   LCD_RST        = 0;  // RESET
   __delay_ms(10);       // Burn some time
   LCD_RST        = 1;  // end reset

   __delay_ms(60);
   
   lcd_putcmd(ST7032_FUNCTION | FUNCTION_8BITS | FUNCTION_1_HIGH | FUNCTION_1_LINE | FUNCTION_nIS);   //function set
   __delay_ms(20);
      
   lcd_putcmd(ST7032_FUNCTION | FUNCTION_8BITS | FUNCTION_1_HIGH | FUNCTION_1_LINE | FUNCTION_nIS);   //function set
   lcd_putcmd(ST7032_FUNCTION | FUNCTION_8BITS | FUNCTION_1_HIGH | FUNCTION_1_LINE | FUNCTION_nIS);   //function set

   // Enter the second page of instructions
   lcd_putcmd(ST7032_FUNCTION | FUNCTION_8BITS | FUNCTION_1_HIGH | FUNCTION_2_LINE | FUNCTION_IS);   //function set
   lcd_putcmd(ST7032_OSC_FREQ | BIAS_1_5 | FREQ_CNTRL(4));   //internal osc frequency
   lcd_putcmd(ST7032_PWR_CONTROL | nICON | BOOST | CONTRAST(2));   //power control
   lcd_putcmd(ST7032_FOLLOWER_ON | FOLLOWER_GAIN(5));   //follower control
   lcd_putcmd(ST7032_CONTRAST(0));   //contrast
   // leave second instruction page
   
//   lcd_putcmd(ST7032_FUNCTION | FUNCTION_8BITS | FUNCTION_1_HIGH | FUNCTION_2_LINE | FUNCTION_nIS);   //function set
   lcd_putcmd(ST7032_DISPLAY_ON | CURSOR_OFF | BLINK_OFF);   //display on
   lcd_putcmd(ST7032_ENTRY_MODE | CURSOR_nSHIFT | DATA_INCREMENT);   //entry mode
   lcd_putcmd(ST7032_CLEAR);   //clear

   __delay_ms(50);
}

/**
 * This function sends a command to the LCD.
 */
void lcd_putcmd(UINT8 Command)
{		
   LCD_CS_OUT = LCD_SELECTED;
   LCD_RS = LCD_RS_COMMAND;
   SPI_TX(Command);
}

/**
 * This function puts a character onto the screen.
 */
void lcd_putchar(const char c)
{
   LCD_CS_OUT = LCD_SELECTED;
   LCD_RS = LCD_RS_DATA;
   SPI_TX(c);
}

void lcd_putascii(unsigned char c)
{
    char i = 0;
	char t = 0;

  	for (i = 0; i < 2; i++)
	{
		t = (0xF0 & c) >> 4; /* Get first nibble */

		/* Convert nibble to ASCII value */
		if (t > 0x9)
			t = t + 0x37;
		else
			t = t + 0x30;

		/* Send character to LCD */
		lcd_putchar(c);

		/* Shift long over for next iteration */
		c = c << 4;
	}
}


/** @}*/
