//
// MICROCHIP SOFTWARE NOTICE AND DISCLAIMER: You may use this software,
// and any derivatives created by any person or entity by or on your
// behalf, exclusively with Microchip’s products. Microchip and its
// licensors retain all ownership and intellectual property rights in
// the accompanying software and in all derivatives hereto.
//
// This software and any accompanying information is for suggestion
// only. It does not modify Microchip’s standard warranty for its
// products. You agree that you are solely responsible for testing the
// software and determining its suitability. Microchip has no obligation
// to modify, test, certify, or support the software.
//
// THIS SOFTWARE IS SUPPLIED BY MICROCHIP "AS IS". NO WARRANTIES,
// WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED TO,
// IMPLIED WARRANTIES OF NON-INFRINGEMENT, MERCHANTABILITY, AND FITNESS
// FOR A PARTICULAR PURPOSE APPLY TO THIS SOFTWARE, ITS INTERACTION WITH
// MICROCHIP’S PRODUCTS, COMBINATION WITH ANY OTHER PRODUCTS, OR USE IN
// ANY APPLICATION.
//
// IN NO EVENT, WILL MICROCHIP BE LIABLE, WHETHER IN CONTRACT, WARRANTY,
// TORT (INCLUDING NEGLIGENCE OR BREACH OF STATUTORY DUTY), STRICT
// LIABILITY, INDEMNITY, CONTRIBUTION, OR OTHERWISE, FOR ANY INDIRECT,
// SPECIAL, PUNITIVE, EXEMPLARY, INCIDENTAL OR CONSEQUENTIAL LOSS,
// DAMAGE, FOR COST OR EXPENSE OF ANY KIND WHATSOEVER RELATED TO THE
// SOFTWARE, HOWSOEVER CAUSED, EVEN IF MICROCHIP HAS BEEN ADVISED OF THE
// // POSSIBILITY OR THE DAMAGES ARE FORESEEABLE. TO THE FULLEST EXTENT
// ALLOWABLE BY LAW, MICROCHIP'S TOTAL LIABILITY ON ALL CLAIMS IN ANY
// WAY RELATED TO THIS SOFTWARE WILL NOT EXCEED THE AMOUNT OF FEES, IF
// ANY, THAT YOU HAVE PAID DIRECTLY TO MICROCHIP FOR THIS SOFTWARE.
//
// MICROCHIP PROVIDES THIS SOFTWARE CONDITIONALLY UPON YOUR ACCEPTANCE
// OF THESE TERMS.
//

/**
 * @file buttons.h
 * @brief This file contains the function prototypes for the button press
 * features. This file also contains defines applicable to the buttons.
 */

#ifndef __BUTTONS_H
#define __BUTTONS_H

typedef enum {
    BTN_UP,
    BTN_DOWN,
    BTN_FALL,
    BTN_RELEASE
} btn_event_t;

//******************************************************/
//		BUTTONS DEFINES
//******************************************************/

// Menu Navigation
#define SW2_PRESSED             sw2_poll() == BTN_RELEASE
#define SW3_PRESSED             sw3_poll() == BTN_RELEASE

#define SW2_HELD                sw2_poll() == BTN_DOWN
#define SW3_HELD                sw3_poll() == BTN_DOWN

#define SW2_FALL                sw2_poll() == BTN_FALL
#define SW3_FALL                sw3_poll() == BTN_FALL

// Number Assignment, Misc.
#define PRESSED                 0
#define UNPRESSED               1
#define DEBOUNCE_COUNTER        0
#define REPEAT_TIME             5
//******************************************************/
//		Buttons Prototypes
//******************************************************/
void buttons_update(void);
void buttons_init(void);
btn_event_t sw2_poll(void);
btn_event_t sw3_poll(void);

#endif  // __BUTTONS_H