/**
 * @file spi.h
 * @brief
 * @{
 */
#ifndef __SPI_H
#define __SPI_H


#define SPI_INTERRUPT_FLAG              SSP1IF
#define SPI_WRITE_COLLISION             SSP1CON1bits.WCOL
#define SPI_OVERFLOW                    SSP1CON1bits.SSPOV
#define SPI_TRANSMIT_REGISTER           SSP1BUF
#define SPI_RECEIVE_REGISTER            SSP1BUF
#define SPI_BUFFER_FULL                 SSP1STATbits.BF




enum spiState
{
	SPI_IDLE,
	SPI_STARTING,
	SPI_TRANSMITING,
	SPI_RECEIVING,
        SPI_STOPPING,
	SPI_ERRORS
};

struct trb
{
	unsigned char cmmd; //correlates to "instruction" in datasheet
	unsigned char txLen;
	unsigned char rxLen;
	const unsigned char *txBuf;
	unsigned char *rxBuf;
	unsigned char retries;
	volatile unsigned char status;
	struct trb *next;
};

/**
 * Transact one byte on the SPI bus.
 *
 * Be sure to manipulate the Chip Select signals before calling.
 */
unsigned char spi_write(unsigned char data);
unsigned char spi_bb_read(void);
void spi_init(unsigned char mode);
/**
 * @brief Initializes the SPI hardware and data structures
 * @param bus i2c bus status
 * @param req i2c bus properties
 */







#endif
/** @}*/