/**
 * @file spi.c
 * @brief All SPI routines
 *
 */

#include "../include/include.h"


void spi_init(unsigned char mode) {

    
    if(mode == Master)
    {
        SSP1CON1 = 0;
        SSP1CON2 = 0;
        SSP1CON3 = 0;
        SSP1STAT = 0;
        
        TRISAbits.TRISA1 = 1;
        TRISCbits.TRISC1 = 1;

        SPI_SCK_DIR = 0;
        SPI_SDO_DIR = 0;

        INTCONbits.GIE = 0;     //disable interrupts


        SSP1CON1bits.CKP = 0; //idle state for clock is a low level
        SSP1STATbits.CKE = 1; //transmit occurs from idle to active state
        SSP1STATbits.SMP = 0; //input data sampled at middle of data output time

        //clock = Fosc/64
        SSP1CON1bits.SSPM0 = 0;
        SSP1CON1bits.SSPM1 = 0;
        SSP1CON1bits.SSPM2 = 0;
        SSP1CON1bits.SSPM3 = 0;
        SSP1CON1bits.SSPEN = 1; //enable MSSP module

    }
    
    if(mode == Slave){
        

        SPI_SCK_DIR = 1;
        SPI_SDI_DIR = 1;

        TRISBbits.TRISB5 = 1;   //CLC4 Decoder Data Input
        TRISCbits.TRISC3 = 1;   //CLC2 Decoder Data Input
        TRISCbits.TRISC1 = 0;
        
        INTCONbits.PEIE = 1;
        INTCONbits.GIE = 1;

        INTCONbits.IOCIE = 1;
        IOCBNbits.IOCBN5 = 1;
        IOCBPbits.IOCBP5 = 1;

    }
    
    
}

unsigned char spi_bb_read(void)
{
    unsigned char i;
    unsigned char byteIn = 0;
    
    for(i=0; i<8; i++) {
        // wait for clock to go low
        while(SPI_SCK);
            //LATCbits.LATC5 = 1;
        // now wait for clock to go high
        while(!SPI_SCK);
            //LATCbits.LATC5 = 0;
        // clock is high, time to latch the data
        if (SPI_SDI)
            byteIn |= (1<<(7-i));
        // if we make it here than we've gone through all 8 bits
    }

    __delay_us(60);
	
	return byteIn;
}

unsigned char spi_write(unsigned char data)
{

    SSP1BUF = data;
    while (!SSP1STATbits.BF); //wait until finished sending
    
    return SSP1BUF; //clear BF flag
}
