/***********************************************************************
 * MICROCHIP SOFTWARE NOTICE AND DISCLAIMER: You may use this software, and
 * any derivatives created by any person or entity by or on your behalf,
 * exclusively with Microchip's products. Microchip and its licensors
 * retain all ownership and intellectual property rights in the
 * accompanying software and in all derivatives hereto.
 *
 * This software and any accompanying information is for suggestion only.
 * It does not modify Microchip's standard warranty for its products. You
 * agree that you are solely responsible for testing the software and
 * determining its suitability. Microchip has no obligation to modify,
 * test, certify, or support the software.
 *
 * THIS SOFTWARE IS SUPPLIED BY MICROCHIP "AS IS". NO WARRANTIES, WHETHER
 * EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED TO, IMPLIED
 * WARRANTIES OF NON-INFRINGEMENT, MERCHANTABILITY, AND FITNESS FOR A
 * PARTICULAR PURPOSE APPLY TO THIS SOFTWARE, ITS INTERACTION WITH
 * MICROCHIP'S PRODUCTS, COMBINATION WITH ANY OTHER PRODUCTS, OR USE IN ANY
 * APPLICATION.
 *
 * IN NO EVENT, WILL MICROCHIP BE LIABLE, WHETHER IN CONTRACT, WARRANTY,
 * TORT (INCLUDING NEGLIGENCE OR BREACH OF STATUTORY DUTY), STRICT
 * LIABILITY, INDEMNITY, CONTRIBUTION, OR OTHERWISE, FOR ANY INDIRECT,
 * SPECIAL, PUNITIVE, EXEMPLARY, INCIDENTAL OR CONSEQUENTIAL LOSS, DAMAGE,
 * FOR COST OR EXPENSE OF ANY KIND WHATSOEVER RELATED TO THE SOFTWARE,
 * HOWSOEVER CAUSED, EVEN IF MICROCHIP HAS BEEN ADVISED OF THE POSSIBILITY
 * OR THE DAMAGES ARE FORESEEABLE. TO THE FULLEST EXTENT ALLOWABLE BY LAW,
 * MICROCHIP'S TOTAL LIABILITY ON ALL CLAIMS IN ANY WAY RELATED TO THIS
 * SOFTWARE WILL NOT EXCEED THE AMOUNT OF FEES, IF ANY, THAT YOU HAVE PAID
 * DIRECTLY TO MICROCHIP FOR THIS SOFTWARE.
 *
 * MICROCHIP PROVIDES THIS SOFTWARE CONDITIONALLY UPON YOUR ACCEPTANCE OF
 * THESE TERMS.
 *************************************************************************/
 /****************************************************************************
 * Revision History
 * Date		Rev	 Author  Comments
 * 09.20.10 1.00         Public Release
 ****************************************************************************/
/** @file m_i2c.c
* @brief This file contains an implementation of I2C Master 
*/

#include <htc.h>

#include "m_i2c.h"

//*************************************************************************
// void i2c_start(void) - I2C Master send start condition on I2C bus
//*************************************************************************
/**
*   @brief This routine is used creates a start condition on I2C bus
*/

void i2c_start(void)
    {
	MSSPCON2bits.SEN = 0x01;	//
	while (MSSPCON2bits.SEN )CLRWDT();
	NOP();						//for Breakpoint
    }

//*************************************************************************
// void i2c_stop(void) - I2C Master send stop condition on I2C bus
//*************************************************************************
/**
*   @brief This routine is used creates a stop condition on I2C bus
*/
void i2c_stop(void)
{
	MSSPCON2bits.PEN = 0x01;	//
	while (MSSPCON2bits.PEN) CLRWDT();
	NOP();						//for Breakpoint
}

//*************************************************************************
// void i2c_sendbyte(unsigned char data) - I2C Master send a byte on I2C bus
//*************************************************************************
/**
*   @brief This routine is used creates a stop condition on I2C bus
*      @param data - byte sent to I2C bus
*/
void i2c_sendbyte(unsigned char data)
{
	MSSPBUF = data;
	while (MSSPSTATbits.R_nW )CLRWDT();
	NOP();						//for Breakpoint
}

//*************************************************************************
// unsigned char i2c_getbyte(void) - I2C Master get a byte on I2C bus
//*************************************************************************
/**
*   @brief This routine is used creates a stop condition on I2C bus
*   @return data - byte from I2C bus
*/
unsigned char i2c_getbyte(void)
{
	MSSPCON2bits.RCEN = 1;
	while (MSSPCON2bits.RCEN) CLRWDT();
	NOP();						//for Breakpoint
	return MSSPBUF;
}


//*************************************************************************
// char i2c_getack(void) - I2C Master check if transaction was acknoledged by I2C Slave
//*************************************************************************
/**
*   @brief This routine check if the sent byte was acknoledged or not by I2C slave
*   	@return  1 - I2C Slave acknoledges the transaction
*   	@return  0 - I2C Slave not acknoledges the transaction (wrong transfer)
*/
//*************************************************************************
char i2c_getack(void)
{

 if(MSSPCON2bits.ACKSTAT) return(0);                      //-- No ACK so return BAD
 else return(1);
}


//*************************************************************************
// void i2c_init(void) - I2C module initialization routine
//*************************************************************************
/**
*   @brief This procedure initializes the microcontrollers peripherals for I2C communication in master mode
*/
//@{
void i2c_init(void)
{
	SDA_TRIS = 1;
	SCL_TRIS = 1;

//	ANSEL_I2C_SDA = 0;
//	ANSEL_I2C_CLK = 0;

	MSSPSTAT = 0X00;
	MSSPADD = 100; // 400kHz @ 16MHz clock - limited  by LCD specs

	MSSPCON1 = 0X28;//	SEN = 1; I2C MASTER 
	MSSPCON2 = 0X00;//

        PIR1bits.SSP1IF = 0;
        PIR2bits.BCL1IF = 0;
}
//@}
