/********************************************************************
 *
 *
 *
 *********************************************************************
 * FileName:         Main.c
 * Dependencies:     See INCLUDES section below
 * Processor:        PIC16F1829
 * Compiler:         HTC
 * Company:          Microchip Technology, Inc.
 *
 * Software License Agreement:
 *
 * The software supplied herewith by Microchip Technology Incorporated
 * (the "Company") for its PICmicro® Microcontroller is intended and
 * supplied to you, the Company's customer, for use solely and
 * exclusively on Microchip PICmicro Microcontroller products. The
 * software is owned by the Company and/or its supplier, and is
 * protected under applicable copyright laws. All rights are reserved.
 * Any use in violation of the foregoing restrictions may subject the
 * user to criminal sanctions under applicable laws, as well as to
 * civil liability for the breach of the terms and conditions of this
 * license.
 *
 * THIS SOFTWARE IS PROVIDED IN AN "AS IS" CONDITION. NO WARRANTIES,
 * WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED
 * TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE COMPANY SHALL NOT,
 * IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR
 * CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.
 *********************************************************************
 * File Description: This project is to create and demo working code with
 *                   the New Peripheral Demo Board. Spicifically the
 *                   the Manchester Encoder and CWG
 * Change History:
 * Author               Date        Version
 * Brian Bailey        7-9-12       1.0A  (Initial Release)
 ********************************************************************/

/**
 *   @file Main.c
 *   @brief
 *   @defgroup
 *
 */

// Manchester Encoder

#include "../include/include.h"
#include "../include/pic16_config.h"

/***********************************************/



/***********************************************/

unsigned char result = 0;
char overflow = 0;

/********************************************************************
 * Function:             Interrupt Service Routine
 *
 * PreCondition: 	Interrupts must be enabled.
 *
 * Side Effects:
 */
/**
 * @ingroup  		ISR
 *
 * @brief    		This function is used for interrupts.
 *
 * @note    		None
 */

/*******************************************************************/
void interrupt isr(void) {
    if (NCO1IF) {
        NCO1IF = 0;
        overflow++;
    }
}

static void main_init() {
    OSCCON = 0x78; //16MHz

    ANSELA = 0x01;
    ANSELB = 0x00;
    ANSELC = 0x00;

    TRISA = 0x00;
    TRISB = 0x00;
    TRISC = 0x00;

    PORTA = 0x00;
    PORTB = 0x00;
    PORTC = 0x00;

    IRLED0_DIR = 0;
    LED1_DIR = 0;

    LED1_OUT = LED_OFF;
    IRLED0_OUT = LED_OFF;

    //Setup pin configurations for Manchester Encoding
    TRISCbits.TRISC4 = 0; //CLC4 Out (Lab 1)
    TRISCbits.TRISC0 = 0; //CLC2 Out (Lab 4)
    TRISCbits.TRISC6 = 1; //CLC3 In from Motor Optics (Lab 4)

    buttons_init();

    //***********************************************
    //Setup ADC for Motor Speed Control (Lab 4)
#ifdef _LAB4
    TRISAbits.TRISA4 = 1; //RA4 = ADC In
    ANSELB = 0x20;
    ADCON0 = 0x0D;
    ADCON1 = 0x20;
    ADCON2 = 0x00;
#endif
    //***********************************************

    __delay_ms(200); //Wait for startup

}


/********************************************************************
 * Function:             Main
 *
 * PreCondition: 	None
 *
 * Side Effects: 	None
 */
/**
 * @ingroup  		Main
 *
 * @brief    		This is the main function
 *
 * @note    		None
 */

/*******************************************************************/
void main(void) {

    unsigned long int motor_speed = 0; //Lab 4
    unsigned char message[20];
    char test = 0;

    main_init();
    __delay_ms(200); //Wait for startup

    //initialize SPI
    spi_init();
    //Initialize LCD
    lcd_init();
    //Display Splash Screen
    Display_Splash();
    //Setup Manchester Encoder
    encoder_init();
    //Setup CWG (Lab 4)
    CWG_Setup();

    while (1) {
        buttons_update(); //constantly poll buttons
        if (SW3_HELD || SW3_FALL) {
            test++;
            //***********************************************
#ifdef _LAB4
            //Capture ADC data (Lab 4)
            ADC_Convert();

            //Convert ADRES to PWM Duty Cycle for CWG (Lab 4)
            PWM1DCH = ADRESH;

            //Measure Motor Speed (Lab 4)
            motor_speed = CLC_Timer();
            //***********************************************
            //            Lab 1 Message
#elif _LAB1
            sprintf(message, "Lab 1...          ");
            send_encoded_message(LCD_Line_1, message);
            __delay_ms(1);
            sprintf(message, "Testing %d    ", test);
            send_encoded_message(LCD_Line_2, message);

            __delay_us(100);
            lcd_clear();
            lcd_goto(0, 0);
            printf("Encoder:");
            lcd_goto(0, 1);
            printf("Testing %d", test);
#endif
            //            Lab 4 Message (Uncomment for Lab 4)
#ifdef _LAB4
            sprintf(message, "Yay...          ");
            send_encoded_message(LCD_Line_1, message);
            __delay_ms(1);
            sprintf(message, "Speed: %d RPM    ", motor_speed);
            send_encoded_message(LCD_Line_2, message);

            __delay_us(100);
            lcd_clear();
            lcd_goto(0, 0);
            printf("Motor Speed:");
            lcd_goto(0, 1);
            printf("%d RPM", motor_speed);
#endif
        }


        __delay_ms(100);

    }
}

void Display_Splash(void) {
    lcd_clear();
    lcd_goto(0, 0);
    printf("Manchester");
    lcd_goto(0, 1);
    printf("Encoder");
}

/********************************************************************
 * Function:             CWG_Setup
 *
 * PreCondition: 	<main.h>
 *
 * Side Effects: 	None
 */
/**
 * @ingroup  		Main
 *
 * @brief    		This function is used to setup the CWG for Motor Control
 *
 * @note    		Lab 4
 */

/*******************************************************************/
void CWG_Setup(void) {

   Setup CWG setup

    //Setup TMR2

   // PR2 = ??;
   // T2CON = ??;

    //Setup PWM
   // PWM1DCH = ??; //0% Duty Cycle
   // PWM1DCL = ??;


    //Setup CWG
   // TRISCbits.TRISC4 = 0;
   // TRISCbits.TRISC5 = 0;

   // CWG1CON1 = ??;
   // CWG1CON2 = ??;
   // CWG1DBR = ??;
   // CWG1DBF = ??;

   // PWM1CON = ??; //Enable PWM
   // CWG1CON0 = ??; //Enable CWG

    //***********************************************

}

/********************************************************************
 * Function:             CLC_Timer
 *
 * PreCondition: 	<main.h>
 *
 * Side Effects: 	None
 */
/**
 * @ingroup  		Main
 *
 * @brief    		This function is used to read the Motor Speed
 *
 * @note    		Lab 4
 */

/*******************************************************************/
unsigned long int CLC_Timer(void) {
    unsigned long int speed = 0;
    char timeout = 0;
    char i = 0;
    char j = 1;
    char k = 0;

    NCO1ACC = 0; // Clear NCO Accumulator

    CLC_Timer_GO = 0; //Start counter on next rising edge
    while (!(CLC3IF || timeout)) //Wait for revolution pulse
    {
        //Timeout function - needed if motor is not spinning
        i++;
        if (!i)
            j++;
        if (!j) {
            k++;
            j = 1;
        }
        if (k >= 3)
            timeout = 1;
    }

    CLC_Timer_GO = 1; //Stop Counting
    CLC3IF = 0; //Clear Interrupt Flag

    //Exit if time out occurs
    if (timeout) {
        speed = 0;
        return speed;
    }

    //Compensate for NCO Timer Overflow
    for (i = 0; i < overflow; i++)
        speed = speed + 0x100000;

    overflow = 0; //Reset Overflow counter

    //Calculate Speed from NCO Accumulator value
    speed = speed + NCO1ACC;
    speed = 480000000 / speed; //=number of clock NCO counts in 1 minute/clocks measured

    return speed;
}

void ADC_Convert(void) {

    //Start Conversion
    GO_nDONE = 1;
    while (GO_nDONE);

    return;

}
/********************************************************************
 * Function:             putch
 *
 * PreCondition: 	<stdio.h>
 *
 * Side Effects: 	None
 */
/**
 * @ingroup  		LCD
 *
 * @brief    		This function is used to enable printf()
 *
 * @note    		None
 */

/*******************************************************************/
void putch(char c) {
    lcd_putchar(c);
}