/**
 * @file spi.c
 * @brief All SPI routines
 *
 */

#include "../include/include.h"

/**
 * @var bus1
 * @brief global variables that are used to init the i2c bus
 */
struct i2cBus bus1 = {
    SPI_IDLE,
    0, // no stopLock
    0, // offset 0
    (void *) 0,     //points to no structure
    &bus1.req
};


void spi_init() {
    char result;
    SSP1CON1 = 0;
    SSP1CON2 = 0;
    SSP1CON3 = 0; 
    SSP1STAT = 0;

        //PR2 = 0x28;             //50kHz signal
        //PR2 = 0xD0;             //9.6kHz signal
        PR2 = 0x50;           //25kHz signal
        
        SPI_SCK_DIR = 0;
        SPI_SDO_DIR = 0;

        INTCONbits.GIE = 0;     //disable interrupts


        SSP1CON1bits.CKP = 0; //idle state for clock is a low level
        SSP1STATbits.CKE = 1; //transmit occurs from idle to active state
        SSP1STATbits.SMP = 0; //input data sampled at middle of data output time

        //clock = Fosc/64
        SSP1CON1bits.SSPM0 = 0;
        SSP1CON1bits.SSPM1 = 0;
        SSP1CON1bits.SSPM2 = 0;
        SSP1CON1bits.SSPM3 = 0;
        SSP1CON1bits.SSPEN = 1; //enable MSSP module

//        __delay_ms(10);
//        result = spi_write(0x00);
//        __delay_ms(10);
//        result = spi_write(0x00);
//        __delay_ms(10);
//        result = spi_write(0x00);
//        __delay_ms(10);
//        result = spi_write(0x00);
//        __delay_ms(10);
//        result = spi_write(0x00);
//        __delay_ms(10);
//        result = spi_write(0x00);
//        __delay_ms(10);
    
}


unsigned char spi_write(unsigned char data)
{

//    T2CON |= 0x04;
    //SSP1STATbits.CKE = 1; //transmit occurs from idle to active state

    CLC4CON ^= 0x80;    

    SSP1BUF = data;
    while (!SSP1STATbits.BF); //wait until finished sending
    
    //SSP1STATbits.CKE = 0; //transmit occurs from idle to active state

    CLC4CON ^= 0x80;
    
    return SSP1BUF; //clear BF flag
}

unsigned char spi_manchester_write(unsigned char data)
{

    //change clock source for manchester encoding
    //clock = TMR2
    SSP1STATbits.CKE = 0; //transmit occurs from idle to active state
    SSP1CON1bits.SSPM0 = 1;
    SSP1CON1bits.SSPM1 = 1;
    //bit bang start bit for manchester decoder
    CLC4POL ^= 0x80;
    
    T2CON |= 0x04;
    
    SSP1BUF = data;
    while (!SSP1STATbits.BF); //wait until finished sending

    CLC4POL ^= 0x80;
    T2CON &= ~0x04;
    //__delay_us(5);
    

    TMR2 = 0x00;

    //change clock source to normal
    //clock = TMR2
    SSP1STATbits.CKE = 1; //transmit occurs from idle to active state
    SSP1CON1bits.SSPM0 = 0;
    SSP1CON1bits.SSPM1 = 0;

    __delay_us(200);        //delay for decoder
    
    return SSP1BUF; //clear BF flag
}
